# -*- coding: utf-8 -*-
#
#       interface/misc/skillsystem.py
#       Copyright 2009 Espen Rønnevik <brightside@quasinet.org>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

from xml.parsers.expat import ParserCreate
from collections import deque

# ------------------------------------------------------------------------------
# BEGIN Group and GroupCollection Class
# ------------------------------------------------------------------------------

class Group(object):

    # 'Special' methods

    def __init__(self, gid, name):

        # 'Private' variables 

        self.__gid = gid
        self.__name = name
        self.__sids = []

    def __len__(self):
        return len(self.__sids)

    # 'Protected' methods

    def _addSid(self, sid):
        self.__sids.append(sid)

    # 'Public' properties

    @property
    def gid(self):
        return self.__gid

    @property
    def name(self):
        return self.__name

    # 'Public' methods

    def getSids(self):
        return tuple(self.__sids)


class GroupCollection(object):

    # 'Special' methods

    def __init__(self):

        # 'Private' variables 

        self.__groups = {}

    def __len__(self):
        return len(self.__groups)

    def __iter__(self):
        return self.__groups.itervalues()

    # 'Protected' methods

    def _addGroup(self, groupID, groupName, *args, **kwargs):
        gob = Group(groupID, groupName)
        self.__groups[groupID] = gob
        return gob

    def _clear(self):
        self.__groups.clear()

    def _addSid(self, gid, sid):
        self.__groups[gid]._addSid(sid)

    # 'Public' methods

    def getGroupByID(self, gid):
        return self.__groups[gid]

    def getGroupByName(self, name):
        for gob in self.__groups.itervalues():
            if gob.name == name:
                return gob

# ------------------------------------------------------------------------------
# END Group and GroupCollection Class
# ------------------------------------------------------------------------------

# ------------------------------------------------------------------------------
# BEGIN Skill and SkillCollection Class
# ------------------------------------------------------------------------------

class Skill(object):

    # 'Special' Methods
    
    def __init__(self, sid, gid, name):

        # 'Private' variables
        self.__sid = sid
        self.__gid = gid
        self.__name = name

        self.__description = ""

        self.__primaryAttribute = None
        self.__secondaryAttribute = None
        self.__rank = 0
        self.__level = 0
        self.__skillPoints = 0

        self.__requiredSkill = {}
        self.__skillBonus = {}

    # 'Protected' methods

    def _setPrimaryAttribute(self, attribute):
        self.__primaryAttribute = attribute

    def _setSecondaryAttribute(self, attribute):
        self.__secondaryAttribute = attribute

    def _addRequiredSkill(self, skillID, level):
        self.__requiredSkill[skillID] = int(level)

    def _addSkillBonus(self, bonustype, value):
        self.__skillBonus[bonustype] = value

    def _setDescription(self, description):
        self.__description = description

    def _setRank(self, rank):
        self.__rank = int(rank)

    def _setLevel(self, level):
        self.__level = int(level)

    def _setSkillPoints(self, skillPoints):
        self.__skillPoints = int(skillPoints)

    # 'Public' properties

    @property
    def sid(self):
        return self.__sid

    @property
    def gid(self):
        return self.__gid

    @property
    def name(self):
        return self.__name

    @property
    def description(self):
        return self.__description

    @property
    def primaryAttribute(self):
        return self.__primaryAttribute

    @property
    def secondayAttribute(self):
        return self.__secondaryAttribute

    @property
    def rank(self):
        return self.__rank

    @property
    def level(self):
        return self.__level

    @property
    def skillPoints(self):
        return self.__skillPoints
            
    @property
    def isTrialRestricted(self):
        return self.__skillbonus.has_key('canNotBeTrainedOnTrial')

    @property
    def iterBonusItems(self):
        return self.__skillBonus.iteritems()

    @property
    def iterRequiredItems(self):
        return self.__requiredSkill.iteritems()

class SkillCollection(object):

    # 'Special' Methods

    def __init__(self):

        # 'Private' variables

        self.__skills = {}

    def __len__(self):
        return len(self.__skills)

    def __iter__(self):
        return self.__skills.itervalues()

    # 'Protected' methods

    def _addSkill(self, typeID, groupID, typeName, *args, **kwargs):
        sob = Skill(typeID, groupID, typeName)
        self.__skills[typeID] = sob
        return sob
 
    def _clear(self):
        self.__skills.clear()

    # 'Public' methods

    def getSkillByID(self, sid):
        return self.__skills[sid]

    def getSkillByName(self, name):
        for sob in self.__skills.itervalues():
            if sob.name == name:
                return sob

# ------------------------------------------------------------------------------
# END Skill and SkillCollection Class
# ------------------------------------------------------------------------------

# ------------------------------------------------------------------------------
# BEGIN SkillTree Class
# ------------------------------------------------------------------------------

class ParseXML(object):

    def __init__(self, xmlfile, attributes):
        self.groups = GroupCollection()
        self.skills = SkillCollection()

        self.xmlElement = deque()
        self.currentSkill = None
        self.attributes = attributes

        parser = ParserCreate()
        parser.StartElementHandler = self.handleBeginElement
        parser.CharacterDataHandler = self.handleAllData
        parser.EndElementHandler = self.handleEndElement
        parser.returns_unicode = False
        parser.buffer_text = True
        parser.ParseFile(open(xmlfile))
        
    def handleBeginElement(self, element, attrs):
        if element == 'rowset':
            self.xmlElement.append(attrs['name'])
        else:
            if element == 'row':
                self.processRows(self.xmlElement[-1], attrs)
            self.xmlElement.append(element)

    def processRows(self, setname, attrs):
        if setname == 'skillGroups':
            self.groups._addGroup(**attrs)

        elif setname == 'skills':
            sob = self.skills._addSkill(**attrs)
            self.groups._addSid(sob.gid, sob.sid)
            self.currentSkill = sob

        elif setname == 'requiredSkills':
            self.currentSkill._addRequiredSkill(attrs['typeID'], attrs['skillLevel'])

        elif setname == 'skillBonusCollection':
            self.currentSkill._addSkillBonus(attrs['bonusType'], attrs['bonusValue'])
                
    def handleAllData(self, data):
        data = data.strip()
        if data:
            self.handleCharacterData(data)

    def handleCharacterData(self, data):
        currentElement = self.xmlElement[-1]

        if currentElement == 'description':
            self.currentSkill._setDescription(data)

        elif currentElement == 'rank':
            self.currentSkill._setRank(data)

        elif currentElement == 'primaryAttribute':
#            attribute = self.attributes._getAttribute(data)
#            self.currentSkill._setPrimaryAttribute(attribute)
            self.currentSkill._setPrimaryAttribute(data)

        elif currentElement == 'secondaryAttribute':
#            attribute = self.attributes._getAttribute(data)
#            self.currentSkill._setSecondaryAttribute(attribute)
            self.currentSkill._setSecondaryAttribute(data)

    def handleEndElement(self, element):
        self.xmlElement.pop()

class SkillTree(object):

    # 'Special' Methods

    def __init__(self, xmlfile, attributes):
        
        parseResult = ParseXML(xmlfile, attributes)

        self.groups = parseResult.groups
        self.skills = parseResult.skills

    def treedump(self):
        print "Dumping Group information:"
        print "--------------------------"
        for g in self.groups:
            print " * %s (%s) contains %d skills" % (g.name, g.gid, len(g))
        print ""
        print "Dumping Skill information:" 
        print "--------------------------"
        for s in self.skills:
            print " * %s (%d) in '%s'" % (s.name, s.rank, self.groups.getGroupByID(s.gid).name)
            print " * Primary: '%s' Secondary: '%s'" % (s.primaryAttribute, s.secondayAttribute)
            print " * Requires skills:"
            for sid, lvl in s.iterRequiredItems:
                print " *  %s level %s" % (self.skills.getSkillByID(sid).name, lvl)
            print " * Gives bonuses (per level):"
            for b, v in s.iterBonusItems:
                print " *  %s: %s" % (b, v)
            print ""

# ------------------------------------------------------------------------------
# END SkillTree Class
# ------------------------------------------------------------------------------

if __name__ == '__main__':
    SkillTree('../../data/skilltree.xml', None).treedump()
